define(['exports', 'jquery', 'react', 'react-dom', 'html-to-react', 'ent', 'lodash', 'aurelia-pal'], function (exports, _jquery, _react, _reactDom, _htmlToReact, _ent, _lodash, _aureliaPal) {
    'use strict';

    Object.defineProperty(exports, "__esModule", {
        value: true
    });
    exports.configure = configure;

    var _jquery2 = _interopRequireDefault(_jquery);

    var _react2 = _interopRequireDefault(_react);

    var _reactDom2 = _interopRequireDefault(_reactDom);

    var _htmlToReact2 = _interopRequireDefault(_htmlToReact);

    var _ent2 = _interopRequireDefault(_ent);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    var components = {};

    function configure(_ref, comps) {
        var aurelia = _ref.aurelia;

        var loader = aurelia.loader;
        components = comps;
    }

    function renderWithReact(markup) {
        var container = document.createElement('div');
        var doneRendering = false;

        function getSanitizedHTMLString() {
            return { __html: markup };
        };

        var processNodeDefinition = new _htmlToReact2.default.ProcessNodeDefinitions(_react2.default);
        var processInstruction = [{
            shouldProcessNode: function shouldProcessNode(node) {
                return true;
            },
            processNode: processNodeDefinition.processDefaultNode
        }];

        var parser = new _htmlToReact2.default.Parser(_react2.default, { lowerCaseTags: false });

        markup = markup.replace('<template>', '<div>');
        markup = markup.replace('</template>', '</div>');
        markup = '<template>' + markup + '</template>';

        var component = parser.parseWithInstructions(markup, function () {
            return true;
        }, processInstruction);
        _reactDom2.default.render(component, container, function () {
            doneRendering = true;
        });

        while (!doneRendering) {
            console.log("Waiting");
        }

        var output = document.createElement('div');

        output.innerHTML = '<template>' + (0, _jquery2.default)(container.childNodes[0].childNodes[0]).html() + '</template>';

        return output;
    }

    _aureliaPal.DOM.createTemplateFromMarkup = function createTemplateFromMarkup(markup) {
        var template = renderWithReact(markup);

        var temp = template.firstElementChild;
        if (!temp || temp.nodeName !== 'TEMPLATE') {
            throw new Error('Template markup must be wrapped in a <template> element e.g. <template> <!-- markup here --> </template>');
        }

        return _aureliaPal.FEATURE.ensureHTMLTemplateElement(temp);
    };

    _htmlToReact2.default.ProcessNodeDefinitions = function (React) {
        function processDefaultNode(node, children, index) {
            var voidElementTags = ['area', 'base', 'br', 'col', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr', 'textarea'];

            if (node.type === 'text') {
                return _ent2.default.decode(node.data);
            } else if (node.type === 'comment') {
                return false;
            }

            var elementProps = {
                is: "",
                key: index
            };

            if (node.attribs) {
                (0, _lodash.forEach)(node.attribs, function (value, key) {
                    switch (key || '') {
                        case 'style':
                            elementProps.style = createStyleJsonFromString(node.attribs.style);
                            break;
                        case 'class':
                            elementProps.className = value;
                            break;
                        default:
                            elementProps[key] = value;
                            break;
                    }
                });
            }
            if ((0, _lodash.includes)(voidElementTags, node.name)) {
                return React.createElement(node.name, elementProps);
            } else {
                var variable = undefined;

                try {
                    variable = eval('(function(components) { return components.' + node.name + '; })(components)');
                } catch (error) {}

                if (typeof variable === 'function') {
                    return React.createElement(variable, elementProps, node.data, children);
                }

                return React.createElement(node.name, elementProps, node.data, children);
            }
        }

        return {
            processDefaultNode: processDefaultNode
        };
    };
});