import $ from 'jquery';
import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import HtmlToReact from 'html-to-react';
import ent from 'ent';
import { camelCase, forEach, includes } from 'lodash';
import { DOM, FEATURE } from 'aurelia-pal';

var components = {};

export function configure({ aurelia }, comps) {
    const loader = aurelia.loader;
    components = comps;
}

function renderWithReact(markup) {
    var container = document.createElement('div');
    var doneRendering = false;

    function getSanitizedHTMLString() {
        return { __html: markup };
    };

    var processNodeDefinition = new HtmlToReact.ProcessNodeDefinitions(React);
    var processInstruction = [{
        shouldProcessNode: function (node) {
            return true;
        },
        processNode: processNodeDefinition.processDefaultNode
    }];

    var parser = new HtmlToReact.Parser(React, { lowerCaseTags: false });

    markup = markup.replace('<template>', '<div>');
    markup = markup.replace('</template>', '</div>');
    markup = '<template>' + markup + '</template>';

    var component = parser.parseWithInstructions(markup, function () {
        return true;
    }, processInstruction);
    ReactDOM.render(component, container, function () {
        doneRendering = true;
    });

    while (!doneRendering) {
        console.log("Waiting");
    }

    var output = document.createElement('div');

    output.innerHTML = '<template>' + $(container.childNodes[0].childNodes[0]).html() + '</template>';

    return output;
}

DOM.createTemplateFromMarkup = function createTemplateFromMarkup(markup) {
    var template = renderWithReact(markup);

    var temp = template.firstElementChild;
    if (!temp || temp.nodeName !== 'TEMPLATE') {
        throw new Error('Template markup must be wrapped in a <template> element e.g. <template> <!-- markup here --> </template>');
    }

    return FEATURE.ensureHTMLTemplateElement(temp);
};

HtmlToReact.ProcessNodeDefinitions = function (React) {
    function processDefaultNode(node, children, index) {
        var voidElementTags = ['area', 'base', 'br', 'col', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr', 'textarea'];

        if (node.type === 'text') {
            return ent.decode(node.data);
        } else if (node.type === 'comment') {
            return false;
        }

        var elementProps = {
            is: "",
            key: index
        };

        if (node.attribs) {
            forEach(node.attribs, function (value, key) {
                switch (key || '') {
                    case 'style':
                        elementProps.style = createStyleJsonFromString(node.attribs.style);
                        break;
                    case 'class':
                        elementProps.className = value;
                        break;
                    default:
                        elementProps[key] = value;
                        break;
                }
            });
        }
        if (includes(voidElementTags, node.name)) {
            return React.createElement(node.name, elementProps);
        } else {
            var variable = undefined;

            try {
                variable = eval('(function(components) { return components.' + node.name + '; })(components)');
            } catch (error) {}

            if (typeof variable === 'function') {
                return React.createElement(variable, elementProps, node.data, children);
            }

            return React.createElement(node.name, elementProps, node.data, children);
        }
    }

    return {
        processDefaultNode: processDefaultNode
    };
};