import $                    from 'jquery';
import React, { Component } from 'react';
import ReactDOM             from 'react-dom';
import HtmlToReact          from 'html-to-react';
import ent                  from 'ent';
import {
    camelCase,
    forEach,
    includes }              from 'lodash';
import {
    DOM,
    FEATURE }               from 'aurelia-pal';

var components = {};

/*
 * Configure the Aurelia plugin.
 */
export function configure({ aurelia }, comps) {
    const loader = aurelia.loader;
    components = comps;
}

/*
 * Puts the markup through the ReactJS library and returns the rendered HTML markup.
 */
function renderWithReact(markup) {
    var container = document.createElement('div');
    var doneRendering = false;

    function getSanitizedHTMLString() { return { __html: markup }; };

    var processNodeDefinition = new HtmlToReact.ProcessNodeDefinitions(React);
    var processInstruction = [
        {
            shouldProcessNode: function (node) {
                return true;
            },
            processNode: processNodeDefinition.processDefaultNode
        }
    ];

    var parser = new HtmlToReact.Parser(React, { lowerCaseTags: false });

    // Add an div element inside the template elements to retrieve the html code later
    markup = markup.replace('<template>', '<div>');
    markup = markup.replace('</template>', '</div>');
    markup = '<template>' + markup + '</template>';

    var component = parser.parseWithInstructions(markup, function () { return true; }, processInstruction);
    ReactDOM.render(component, container, function () {
        doneRendering = true;
    });

    while (!doneRendering) {
        console.log("Waiting");
    }

    var output = document.createElement('div');

    // ToDo: Remove the dependency on jQuery.
    output.innerHTML = '<template>' + $(container.childNodes[0].childNodes[0]).html() + '</template>';

    return output;
}

/*
 * Monkey patch'd functionality from Aurelia. Renders the markup through ReactJS.
 */
DOM.createTemplateFromMarkup = function createTemplateFromMarkup(markup) {
    var template = renderWithReact(markup);
    
    var temp = template.firstElementChild;
    if (!temp || temp.nodeName !== 'TEMPLATE') {
        throw new Error('Template markup must be wrapped in a <template> element e.g. <template> <!-- markup here --> </template>');
    }

    return FEATURE.ensureHTMLTemplateElement(temp);
}

/*
 * Monkey patch'd this method so it sees locally imported components
 * Also made sure custom elements are allowed.
 */
HtmlToReact.ProcessNodeDefinitions = function (React) {
    function processDefaultNode(node, children, index) {
        var voidElementTags = ['area', 'base', 'br', 'col', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr', 'textarea'];

        if (node.type === 'text') {
            return ent.decode(node.data);
        } else if (node.type === 'comment') {
            // FIXME: The following doesn't work as the generated HTML results in "&lt;!--  This is a comment  --&gt;"
            //return '<!-- ' + node.data + ' -->';
            return false;
        }

        var elementProps = {
            is: "",     // I know, it's a dick move but at least react doesn't bother about its whitelist anymore 
            key: index
        };

        // Process attributes
        if (node.attribs) {
            forEach(node.attribs, function (value, key) {
                switch (key || '') {
                    case 'style':
                        elementProps.style = createStyleJsonFromString(node.attribs.style);
                        break;
                    case 'class':
                        elementProps.className = value;
                        break;
                    default:
                        elementProps[key] = value;
                        break;
                }
            });
        }
        if (includes(voidElementTags, node.name)) {
            return React.createElement(node.name, elementProps);
        } else {
            // Check whether we can instantiate a custom control.
            var variable = undefined;

            try {
                variable = eval('(function(components) { return components.' + node.name + '; })(components)');
            } catch (error) { }

            if (typeof variable === 'function') {
                return React.createElement(variable, elementProps, node.data, children);
            }

            return React.createElement(node.name, elementProps, node.data, children);
        }
    }

    return {
        processDefaultNode: processDefaultNode
    };
};
